use reqwest::header::{HeaderMap, HeaderValue};
use serde::{Deserialize, Serialize};

#[allow(dead_code)]
pub struct GiteaClient {
    url: String,
    token: Option<String>,
    pub allow_insecure: bool,
}

const APP_USER_AGENT: &'static str =
    concat!(env!("CARGO_PKG_NAME"), "/", env!("CARGO_PKG_VERSION"),);

impl GiteaClient {
    pub fn new(url: impl Into<String>, token: Option<impl Into<String>>) -> Self {
        Self {
            url: url.into(),
            token: token.map(|t| t.into()),
            allow_insecure: false,
        }
    }

    fn create_client(&self) -> anyhow::Result<reqwest::blocking::Client> {
        let cb = reqwest::blocking::ClientBuilder::new();
        let mut header_map = HeaderMap::new();
        if let Some(token) = &self.token {
            header_map.insert(
                "Authorization",
                HeaderValue::from_str(format!("token {}", token).as_str())?,
            );
        }

        let client = cb
            .user_agent(APP_USER_AGENT)
            .default_headers(header_map)
            .danger_accept_invalid_certs(self.allow_insecure)
            .build()?;

        Ok(client)
    }

    pub fn connect(&self, owner: impl Into<String>, repo: impl Into<String>) -> anyhow::Result<()> {
        let client = self.create_client()?;

        let request = client
            .get(format!(
                "{}/api/v1/repos/{}/{}",
                &self.url.trim_end_matches("/"),
                owner.into(),
                repo.into()
            ))
            .build()?;

        let resp = client.execute(request)?;

        if !resp.status().is_success() {
            resp.error_for_status()?;
        }

        Ok(())
    }

    pub fn get_tags(
        &self,
        owner: impl Into<String>,
        repository: impl Into<String>,
    ) -> anyhow::Result<Vec<Tag>> {
        todo!()
    }

    pub fn get_commits_since(
        &self,
        owner: impl Into<String>,
        repository: impl Into<String>,
        since_sha: impl Into<String>,
    ) -> anyhow::Result<Vec<Tag>> {
        todo!()
    }
}

#[derive(Clone, Debug, Deserialize, Serialize)]
pub struct Tag {}
